#include "triangle.hpp"

#include <iostream>

#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp"

Triangle::Triangle(GLuint shaderProgram)
    : _shaderProgram(shaderProgram)
{
    glGenVertexArrays(1, &_vao);
    glGenBuffers(1, &_vbo);

    glBindVertexArray(_vao);

    glBindBuffer(GL_ARRAY_BUFFER, _vbo);
    glBufferData(GL_ARRAY_BUFFER, _vertices.size() * sizeof(float), &_vertices[0], GL_STATIC_DRAW);

    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), (void*)0);
    glEnableVertexAttribArray(0);

    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glBindVertexArray(0);
}

void Triangle::render(float time)
{
    // Calculate matrices
    float left = -1.0, right = 1.0, bottom = -1.0, top = 1.0, near = -1.0, far = 1.0;
    glm::mat4 projection = glm::ortho(left, right, bottom, top, near, far);

    constexpr float angle = glm::radians(50.0);
    glm::vec3 axis = glm::vec3(0.0, 1.0, 0.0);
    glm::mat4 model = glm::rotate(glm::mat4(1.0), angle * time, axis);

    glm::mat4 view = glm::mat4(1.0);

    glm::mat4 mvp = projection * model * view;

	glUseProgram(_shaderProgram);

    GLint mvpLocation = getShaderUniformLocation("_ModelViewProjectionMatrix");
    glUniformMatrix4fv(mvpLocation, 1, GL_FALSE, &mvp[0][0]);

	GLint timeLocation = getShaderUniformLocation("_Time");
	float timeValue = time;
	glUniform1f(timeLocation, timeValue);

	glBindVertexArray(_vao);
	glBindBuffer(GL_ARRAY_BUFFER, _vbo);
	glDrawArrays(GL_TRIANGLES, 0, 3);
}

GLint Triangle::getShaderUniformLocation(const std::string& uniformName)
{
    GLint location = glGetUniformLocation(_shaderProgram, uniformName.c_str());
    if (location == -1)
    {
        std::cerr << "Could not find uniform: " << uniformName << std::endl;
        return -1;
    }

    return location;
}

Triangle::~Triangle()
{
    glDeleteVertexArrays(1, &_vao);
    glDeleteBuffers(1, &_vbo);
}